/*

    What does it do?
    Turns query like "import library/bye" into an array of possible paths that the the file might be at
    "/home/ubuntu/assets/dna/library/bye.dna"
    "/home/ubuntu/assets/dna/library/bye.js"

    Usage:

    require("./import/query parser")(

        // of form: @import dna/homepage@
        query,

        // base directories to append query to
        sub_module["import paths"],

        // response with the path of the dna module
        function(dna_path) {}
    );
*/

/*
    query:  [
        "import",
        "dna/homepage"
    ],
    path:   "/home/ubuntu/webserver/resources/"

    output: "/home/ubuntu/webserver/resources/dna/homepage.dna"
*/

var possible_extension = [
    ".dna",
    ".js"
];


function QueryParser(query, import_paths, output) {
    var instance = this;

    // require("log")(query);
    // require("log")(path);

    this.query = query;
    this.cursor = 1;
    this.import_paths = import_paths;

    // for export
    var possible_paths = [];
    this["get filename"]();

    for (var x=0; x<instance.import_paths.length; x++) {
        var possible_path = instance.import_paths[x] + instance.query_path;
        possible_paths.push(possible_path)
    }

    output(possible_paths);
}
QueryParser.o = QueryParser.prototype = {

    // returns filename
    "get filename": function() {
        var query = this.query;
        var cursor = this.cursor;

        var filename = query[cursor];
        cursor++;

        while (cursor < query.length) {
            if (query[cursor] === "from") {
                cursor++;
                break;
            }
            else {
                filename += " " + query[cursor];
                cursor++;
            }
        }

        // possibly: dna/homepage.dna
        this.query_path = filename + ".dna";
        this.cursor = cursor;

        return this;
    },
    "get location": function() {
        var query = this.query; // []
        var cursor = this.cursor;

        var base = "";
        var extension = "";

        // look for base
        while (cursor < query.length) {
            var firstSlash = query[cursor].indexOf("/");
            if (firstSlash === -1) {
                base += query[cursor];
                cursor++;
            }
            else {
                base += query[cursor].substring(0, firstSlash);
                extension += query[cursor].substring(firstSlash);
                cursor++;
                break;
            }
        }

        // get rest of extension
        while (cursor < query.length) {
            if (extension.length === 0) {
                extension += query[cursor];
            }
            else {
                extension += " " + query[cursor];
            }

            cursor++;
        }

        //this.extension = extension;
        this.base = base;
        this.cursor = cursor;

        return this;
    }
};
function invoker(query, import_paths, output) {
    return new QueryParser(query, import_paths, output);
}

module.exports = invoker;
